@@ -8,11 +8,13 @@ cd "${headers_dir}" || exit 1
 ## Do not abort compilation at header include time
 if grep '^#error "No function renaming possible"' sys/cdefs.h >/dev/null 2>/dev/null; then
 	awk '
-/#error "No function renaming possible"/{
+/^#error "No function renaming possible"/{
 	print "#define __RENAME(x) no renaming on this platform"
 	next
 }
 {print}
+
+/^#warning /{ next }
 	' sys/cdefs.h > sys/cdefs.h.new
 	cat sys/cdefs.h.new > sys/cdefs.h
 	rm -f sys/cdefs.h.new
@@ -30,6 +32,10 @@ if grep '__STDC_HOSTED__' stdint.h >/dev/null 2>/dev/null && grep '_GCC_WRAP_STD
 	echo '#include_next <stdint.h>' > stdint.h
 fi
 
+if grep '__CLANG_LIMITS_H' limits.h >/dev/null 2>/dev/null; then
+	echo '#include_next <limits.h>' > limits.h
+fi
+
 # MUSL libc expects GCC
 if grep ' __builtin_va_list ' bits/alltypes.h >/dev/null 2>/dev/null; then
 	sed 's@ __builtin_va_list @ char * @' bits/alltypes.h > bits/alltypes.h.new
