@@ -128,6 +128,260 @@ namespace eval tcc4tcl {
 		unset $handle
 	}
 
+	proc _proc {handle cname adefs rtype body args} {
+		# Convert body into a C-style string
+		binary scan $body H* cbody
+		set cbody [regsub -all {..} $cbody {\\x&}]
+
+		# Parse optional arguments
+		foreach {argname argval} $args {
+			switch -- $argname {
+				"-error" {
+					set returnErrorValue $argval
+				}
+			}
+		}
+
+		# Argument definitions (in C style) initialization
+		set adefs_c [list]
+
+		# Names of all arguments initialization
+		set args [list]
+
+		# Determine if one of the arguments is a Tcl_Interp*, if not
+		# then we will need to create our own Tcl interpreter for
+		# local use
+		set newInterp 1
+		foreach {type var} $adefs {
+			if {$type == "Tcl_Interp*"} {
+				set newInterp 0
+				set interp_name $var
+
+				break
+			}
+		}
+
+		# Create the C-style argument definition
+		## Create a list of all arguments
+		foreach {type var} $adefs {
+			# Update definition of types
+			lappend adefs_c [list $type $var]
+
+			# Note the type for this variable
+			set types($var) $type
+
+			# The Tcl interpreter is not added to the list of Tcl arguments
+			if {$type == "Tcl_Interp*"} {
+				continue
+			}
+
+			# Update the list of arguments to pass to Tcl
+			lappend args $var
+		}
+
+		## Convert that list into something we can use in a C prototype
+		if {[llength $adefs_c] == 0} {
+			set adefs_c "void"
+		} else {
+			set adefs_c [join $adefs_c {, }]
+		}
+
+		# Determine actual C return type:
+		switch -- $rtype {
+			"ok" {
+				set rtype_c "int"
+			}
+			default {
+				set rtype_c $rtype
+			}
+		}
+
+		# Determine how to return in failure
+		if {$rtype != "void"} {
+			if {[info exists returnErrorValue]} {
+				set return_failure "return(${returnErrorValue})"
+			} else {
+				switch -- $rtype {
+					int - long - Tcl_WideInt {
+						set return_failure "return(-1)"
+					}
+					ok {
+						set return_failure "return(TCL_ERROR)"
+					}
+					double - float {
+						set return_failure "return(($rtype) ((($rtype) 1.0) / (($rtype) 0.0)))"
+					}
+					default {
+						set return_failure "return(NULL)"
+					}
+				}
+			}
+		} else {
+			set return_failure "return"
+		}
+
+		# Define the C function
+		_ccode $handle "$rtype_c $cname\($adefs_c) \{"
+
+		## Define the Tcl return value checking variable
+		_ccode $handle "    int tclrv;"
+
+		## If the interpreters return value is relevant, create a variable to store it
+		if {$rtype != "ok" && $rtype != "void"} {
+			_ccode $handle "    Tcl_Obj *rv_interp;"
+		}
+
+		## If we are returning a value, declare a variable for that
+		if {$rtype != "void"} {
+			_ccode $handle "    $rtype_c rv;"
+		}
+
+		## If we need to create a new interpreter, do so
+		if {$newInterp} {
+			set interp_name "ip"
+			_ccode $handle "    Tcl_Interp *${interp_name};"
+		}
+
+		# Declare Tcl_Obj variables
+		_ccode $handle "    Tcl_Obj *_[join $args {, *_}];"
+
+		_ccode $handle ""
+
+		# Create a new interp if needed, otherwise create a temporary procedure
+		if {$newInterp} {
+			_ccode $handle "    ${interp_name}  = Tcl_CreateInterp();"
+			_ccode $handle "    if (!${interp_name}) $return_failure;"
+			_ccode $handle ""
+
+			set procname ""
+		} else {
+			set procname "::tcc4tcl::tmp::proc[clock clicks]"
+			set cbody "namespace eval ::tcc4tcl {}; namespace eval ::tcc4tcl::tmp {}; proc ${procname} {$args} { $cbody }"
+		}
+
+		# Process all arguments
+		foreach arg $args {
+			set type $types($arg)
+			switch -- $type {
+				int - long - Tcl_WideInt - float - double {
+					switch -- $type {
+						float {
+							set convCmd Double
+						}
+						Tcl_WideInt {
+							set convCmd WideInt
+						}
+						default {
+							set convCmd [string totitle $type]
+						}
+					}
+
+					_ccode $handle "    _$arg = Tcl_New${convCmd}Obj($arg);"
+					_ccode $handle "    if (!_$arg) $return_failure;"
+				}
+				char* {
+					if {[info exists types(${arg}_MemberCount)] && [info exists types(${arg}_MemberLength)]} {
+						_ccode $handle "    _$arg = Tcl_NewByteArrayObj($arg, ${arg}_MemberCount * ${arg}_MemberLength);"
+					} elseif {[info exists types(${arg}_Length)]} {
+						_ccode $handle "    _$arg = Tcl_NewByteArrayObj($arg, ${arg}_Length);"
+					} else {
+						_ccode $handle "    _$arg = Tcl_NewStringObj($arg, -1);"
+					}
+				}
+				Tcl_Obj* {
+					_ccode $handle "    _$arg = $arg;"
+				}
+				default {
+					return -code error "Unknown type: $type"
+				}
+			}
+
+			# If we don't have a procedure to call, set the variables locally
+			if {$procname == ""} {
+				_ccode $handle "    if (!Tcl_ObjSetVar2(${interp_name}, Tcl_NewStringObj(\"${arg}\", -1), NULL, _$arg, 0)) $return_failure;"
+			}
+		}
+		_ccode $handle ""
+
+		# Evaluate script
+		if {$procname != ""} {
+			_ccode $handle "    static int proc_defined = 0;"
+			_ccode $handle "    if (proc_defined == 0) \{"
+			_ccode $handle "        proc_defined = 1;"
+			set extra_space "    "
+		} else {
+			set extra_space ""
+		}
+
+		_ccode $handle "${extra_space}    tclrv = Tcl_Eval($interp_name, \"$cbody\");"
+		_ccode $handle "${extra_space}    if (tclrv != TCL_OK && tclrv != TCL_RETURN) $return_failure;"
+
+		if {$procname != ""} {
+			_ccode $handle "    \}"
+			set i 0
+			_ccode $handle "    Tcl_Obj *objv\[[expr {[llength $args] + 1}]\];"
+			_ccode $handle "    objv\[$i\] = Tcl_NewStringObj(\"$procname\", -1);"
+			foreach arg $args {
+				incr i
+				_ccode $handle "    objv\[$i\] = _$arg;"
+			}
+			_ccode $handle "    tclrv = Tcl_EvalObjv($interp_name, [expr {[llength $args] + 1}], objv, 0);"
+		}
+		_ccode $handle "    if (tclrv != TCL_OK && tclrv != TCL_RETURN) $return_failure;"
+		_ccode $handle ""
+
+		# Handle return value
+		if {$rtype != "ok" && $rtype != "void"} {
+			_ccode $handle "    rv_interp = Tcl_GetObjResult(${interp_name});"
+		}
+
+		switch -- $rtype {
+			void { }
+			ok {
+				_ccode $handle "    rv = TCL_OK;"
+			}
+			int {
+				_ccode $handle "    if (Tcl_GetIntFromObj(ip, rv_interp, &rv) != TCL_OK) $return_failure;"
+			}
+			long {
+				_ccode $handle "    if (Tcl_GetLongFromObj(ip, rv_interp, &rv) != TCL_OK) $return_failure;"
+			}
+			Tcl_WideInt {
+				_ccode $handle "    if (Tcl_GetWideIntFromObj(ip, rv_interp, &rv) != TCL_OK) $return_failure;"
+			}
+			float {
+				_ccode $handle "    {"
+				_ccode $handle "        double t;"
+				_ccode $handle "        if (Tcl_GetDoubleFromObj(ip, rv_interp, &t) != TCL_OK) $return_failure;"
+				_ccode $handle "        rv = (float) t;"
+				_ccode $handle "    }"
+			}
+			double {
+				_ccode $handle "    if (Tcl_GetDoubleFromObj(ip, rv_interp, &rv) != TCL_OK) $return_failure;"
+			}
+			char* {
+				_ccode $handle "    rv = Tcl_GetString(rv_interp);"
+			}
+			Tcl_Obj* {
+				_ccode $handle "    rv = rv_interp;"
+			}
+		}
+
+		# Cleanup created interp if needed
+		if {$newInterp} {
+			_ccode $handle "    Tcl_DeleteInterp(${interp_name});"
+		}
+
+		# Return value
+		_ccode $handle ""
+		if {$rtype != "void"} {
+			_ccode $handle "    return(rv);"
+		} else {
+			_ccode $handle "    return;"
+		}
+		_ccode $handle "\}"
+	}
+
 	proc _go {handle {outputOnly 0}} {
 		variable dir
 
@@ -485,4 +739,4 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""} {includePrototype 0
 
 namespace eval tcc4tcl {namespace export cproc new}
 
-package provide tcc4tcl "0.22"
+package provide tcc4tcl "0.23"
