@@ -82,6 +82,7 @@ $handle delete
 if {[info exists ::env(TCC4TCL_TEST_RUN_NATIVE)]} {
 	set handle [tcc4tcl::new]
 	$handle cwrap curl_version {} vstring
+	$handle add_library_path [::tcl::pkgconfig get libdir,runtime]
 	$handle add_library_path /usr/lib64
 	$handle add_library_path /usr/lib
 	$handle add_library_path /usr/lib32
@@ -105,10 +106,11 @@ if {[info exists ::env(TCC4TCL_TEST_RUN_NATIVE)] && $::tcl_platform(os) != "Darw
 	file delete $tmpfile
 	set handle [tcc4tcl::new $tmpfile "myPkg 0.1"]
 	$handle cproc ext_add {int a int b} long { return(a+b); }
+	$handle add_library_path [::tcl::pkgconfig get libdir,runtime]
 	$handle add_library_path /usr/lib64
 	$handle add_library_path /usr/lib
 	$handle add_library_path /usr/lib32
-	$handle add_library tclstub8.5
+	$handle add_library tclstub${::tcl_version}
 	$handle go
 	load $tmpfile myPkg
 	puts [ext_add 1 42]
@@ -137,6 +139,7 @@ if {[info exists ::env(TCC4TCL_TEST_RUN_NATIVE)]} {
 		return(TCL_OK);
 	}
 	$handle add_include_path /usr/include
+	$handle add_library_path [::tcl::pkgconfig get libdir,runtime]
 	$handle add_library_path /usr/lib64
 	$handle add_library_path /usr/lib
 	$handle add_library_path /usr/lib32
@@ -145,3 +148,34 @@ if {[info exists ::env(TCC4TCL_TEST_RUN_NATIVE)]} {
     
 	curl_fetch http://rkeene.org/
 }
+
+set handle [tcc4tcl::new]
+$handle proc callToTcl {Tcl_Interp* ip int a int b} int {
+	set retval [expr {$a + $b}]
+
+	return $retval
+}
+$handle cwrap callToTcl {Tcl_Interp* ip int a int b} int
+$handle go
+if {[callToTcl 3 5] != 8} {
+	error "3 + 5 is 8, not [callToTcl 3 5]"
+}
+
+set handle [tcc4tcl::new]
+$handle proc callToTcl1 {int x} float {
+	return 0.1
+}
+$handle cwrap callToTcl1 {int x} float
+$handle go
+puts [callToTcl1 3]
+
+set handle [tcc4tcl::new]
+$handle proc callToTclBinary {char* blob int blob_Length} ok {
+	puts "Blob: $blob ([string length $blob])"
+}
+$handle cproc callToTclBinaryWrapper {} void {
+	callToTclBinary("test\x00test", 9);
+}
+puts [$handle code]
+$handle go
+callToTclBinaryWrapper
